/*
 * Copyright (c) 2021-2023 Datalayer, Inc.
 *
 * MIT License
 */
import { createStore } from 'zustand/vanilla';
import { useStore } from 'zustand';
import { cmdIds } from './NotebookCommands';
export const notebookStore = createStore((set, get) => ({
    notebooks: new Map(),
    setNotebooks: (notebooks) => set((state) => ({ notebooks })),
    selectNotebook: (uid) => {
        return get().notebooks.get(uid);
    },
    selectNotebookModel: (uid) => {
        if (get().notebooks.get(uid)) {
            return {
                model: get().notebooks.get(uid)?.model,
                changed: get().notebooks.get(uid)?.model?.contentChanged,
            };
        }
        return undefined;
    },
    selectKernelStatus: (uid) => {
        return get().notebooks.get(uid)?.kernelStatus;
    },
    selectActiveCell: (uid) => {
        return get().notebooks.get(uid)?.activeCell;
    },
    selectNotebookPortals: (uid) => {
        return get().notebooks.get(uid)?.portals;
    },
    selectSaveRequest: (uid) => {
        return get().notebooks.get(uid)?.saveRequest;
    },
    selectNotebookPortalDisplay: (uid) => {
        return get().notebooks.get(uid)?.portalDisplay;
    },
    run: (uid) => { get().notebooks.get(uid)?.adapter?.commands.execute(cmdIds.run); },
    runAll: (uid) => { get().notebooks.get(uid)?.adapter?.commands.execute(cmdIds.runAll); },
    interrupt: (uid) => { get().notebooks.get(uid)?.adapter?.commands.execute(cmdIds.interrupt); },
    insertAbove: (mutation) => {
        get().notebooks.get(mutation.uid)?.adapter?.setDefaultCellType(mutation.cellType);
        get().notebooks.get(mutation.uid)?.adapter?.insertAbove(mutation.source);
    },
    insertBelow: (mutation) => {
        get().notebooks.get(mutation.uid)?.adapter?.setDefaultCellType(mutation.cellType);
        get().notebooks.get(mutation.uid)?.adapter?.insertBelow(mutation.source);
    },
    delete: (uid) => { get().notebooks.get(uid)?.adapter?.commands.execute(cmdIds.deleteCells); },
    changeCellType: (mutation) => {
        get().notebooks.get(mutation.uid)?.adapter?.changeCellType(mutation.cellType);
    },
    save: (mutation) => {
        get().notebooks.get(mutation.uid)?.adapter?.commands.execute(cmdIds.save);
        const notebooks = get().notebooks;
        const notebook = notebooks.get(mutation.uid);
        if (notebook) {
            notebook.saveRequest = mutation.date;
            set((state) => ({ notebooks }));
        }
    },
    reset: () => set((state) => ({ notebooks: new Map() })),
    update: (update) => {
        const notebooks = get().notebooks;
        let notebook = notebooks.get(update.uid);
        if (notebook) {
            notebook = { ...notebook, ...update.partialState };
            set((state) => ({ notebooks }));
        }
        else {
            notebooks.set(update.uid, {
                adapter: update.partialState.adapter,
                portals: [],
            });
            set((state) => ({ notebooks }));
        }
    },
    activeCellChange: (cellModelUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(cellModelUid.uid);
        if (notebook) {
            notebook.activeCell = cellModelUid.cellModel;
            set((state) => ({ notebooks }));
        }
    },
    modelChange: (notebookModelUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(notebookModelUid.uid);
        if (notebook) {
            notebook.model = notebookModelUid.notebookModel;
            set((state) => ({ notebooks }));
        }
    },
    notebookChange: (notebookChangeUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(notebookChangeUid.uid);
        if (notebook) {
            notebook.notebookChange = notebookChangeUid.notebookChange;
            set((state) => ({ notebooks }));
        }
    },
    kernelStatusChanged: (kernelStatusUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(kernelStatusUid.uid);
        if (notebook) {
            notebook.kernelStatus = kernelStatusUid.kernelStatus;
            set((state) => ({ notebooks }));
        }
    },
    changeKernel: (kernelChange) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(kernelChange.uid);
        if (notebook) {
            notebook.adapter?.assignKernel(kernelChange.kernel);
            set((state) => ({ notebooks }));
        }
    },
    addPortals: (portalsUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalsUid.uid);
        if (notebook) {
            notebook.portals = notebook.portals.concat(portalsUid.portals);
            set((state) => ({ notebooks }));
        }
    },
    dispose: (uid) => {
        const notebooks = get().notebooks;
        notebooks.delete(uid);
        set((state) => ({ notebooks }));
    },
    setPortals: (portalsUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalsUid.uid);
        if (notebook) {
            notebook.portals = portalsUid.portals;
            set((state) => ({ notebooks }));
        }
    },
    setPortalDisplay: (portalDisplayUid) => {
        const notebooks = get().notebooks;
        const notebook = notebooks.get(portalDisplayUid.uid);
        if (notebook) {
            notebook.portalDisplay = portalDisplayUid.portalDisplay;
            set((state) => ({ notebooks }));
        }
    },
}));
export function useNotebookStore(selector) {
    return useStore(notebookStore, selector);
}
export default useNotebookStore;
//# sourceMappingURL=NotebookState.js.map